/**
 * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
 */
function RelatorioConsolidadoDeLeadsController(CRM_Utilities, templateManager) {
    
    /**
     * Define os valores padrões dos campos que não podem ter o default value
     *inserido diretamente no JSON e abre o filtro.
     */
    this.widgetOnEnterOnce = function(widget) {
        // Como o operador logado no campo de operador responsável no filtro
        widget.currentRow.NRPARCRESP = [templateManager.LOGGEDOPERATOR.NRPARCNEGOCIO];
        
        // Define o valor padrão do campo de intervalo de tempo no filtro entre
        widget.currentRow.DTINCLUSAO = {
            //um mês da data atual
            start: CRM_Utilities.subtractDateByMonth(moment(), 1),
            period: undefined,
            //e o dia de hoje
            end: CRM_Utilities.getCurrentDate()
        }
        
        // Abre o filtro e define os valores padrões dos campos definidos no JSON
        CRM_Utilities.openFilterOnEnter(widget, widget.currentRow, ['IDATIVO',
            'NMOPERRESP', 'IDSTATUSCAN', 'AGRUPADOR']);
    };
    
    /**
     * Cria as variáveis que serão utilizadas na query com base nos valores
     *definidos no filtro e os adiciona ao data source filter da widget.
     */
    this.definirParametrosDoFiltro = function(args) {
        // Pega os valores inseridos pelo usuário nos campos do filtro
        var filterRow = args.e.preference.FILTER_ROW;
        var widget = args.owner;
        
        // Como os parâmetros que serão utilizados na query são adicionados ao
        //data source filter, é preciso remover aqueles que foram definidos na
        //consulta anterior
        var dataSourceFilter = this.removerFiltrosNaoFixos(widget);
        
        var operadorFiltro = filterRow.NMOPERRESP;
        var leadFiltro = filterRow.NMCANDIDATO;
        var intervalFiltro = filterRow.DTINCLUSAO;
        var ativoFiltro = filterRow.IDATIVO;
        var statusFiltro = filterRow.IDSTATUSCAN;
        
        // Cria o parâmetro do operador.
        //Ex: se os operadores selecionados forem [203040, 473569], ele será
        //definido como 203040_473569_
        if(operadorFiltro) {
            var nrOperadores = this.arrayParaString(operadorFiltro);
            dataSourceFilter = this.criarEAdicionarParametro('P_NRPARCOPER',
                nrOperadores, dataSourceFilter);
        }
        
        // O mesmo se aplica a esse campo
        if(leadFiltro) {
            var nrLeads = this.arrayParaString(leadFiltro);
            dataSourceFilter = this.criarEAdicionarParametro('P_NRLEADS',
                nrLeads, dataSourceFilter);
        }
        
        // E esse também
        if(statusFiltro) {
            var idStatus = this.arrayParaString(statusFiltro);
            dataSourceFilter = this.criarEAdicionarParametro('P_IDSTATUS',
                idStatus, dataSourceFilter);
        }
        
        // E esse... não. Ele cria dois parâmetros: um pra pro início do período
        //a ser filtrado e outro para o fim.
        if(intervalFiltro) {
            var iniData = intervalFiltro.start;
            var fimData = intervalFiltro.end;
            
            dataSourceFilter = this.criarEAdicionarParametro('P_INIDATA',
                iniData, dataSourceFilter);
                
            dataSourceFilter = this.criarEAdicionarParametro('P_FIMDATA',
                fimData, dataSourceFilter);
        }
        
        if(ativoFiltro) {
            dataSourceFilter = this.criarEAdicionarParametro('P_IDATIVO',
                ativoFiltro, dataSourceFilter);
        }
        
        // Atualiza o data source filter da widget.
        widget.dataSourceFilter = dataSourceFilter;
    };
    
    // Transforma um array de valores em string, colocando um "_" ao final de
    //cada valor. ['Bolo', 'é', 'muito', 'bom'] => 'Bolo_é_muito_bom_'
    this.arrayParaString = function(arrayToConvert) {
        var concatString = arrayToConvert.reduce((concat, valueToAdd) => {
            return concat + valueToAdd + '_';
        }, '');
        
        return concatString;
    };
    
    // Cria um novo filtro com base nos valores passados por parâmetro e o
    //adiciona ao data source filter da widget.
    this.criarEAdicionarParametro = function(nomeParam, valorParam,
        dataSourceFilter) {
        
        var novoParam = {
            name: nomeParam,
            // Como se trata de um parâmetro(variável) a ser utilizado na query,
            //o operator definido não faz diferença
            operator: '=',
            value: valorParam,
            fromFilter: true
        };
        
        // Adiciona o novo parâmetro ao data source filter
        dataSourceFilter.push(novoParam);
        
        return dataSourceFilter;
    };
    
    this.removerFiltrosNaoFixos = function(widget) {
        var dataFilter = [...widget.dataSourceFilter];
        var newFilter;
        
        if(!dataFilter) return [];
        
        newFilter = dataFilter.filter(x => x.fixed === true);
        
        return newFilter;
    };
    
    /**
     * Deixa alguns campos do filtro como read-only e remove seus valores da
     *current row com base no agrupador escolhido.
     */
    this.atualizarCamposPeloAgrupador = function(widget) {
        var currentRow = widget.currentRow;
        var agrupadoPorLead = currentRow.AGRUPADOR == 'L';
        
        // Esses campos serão read-only se o resultado for agrupado por Operador.
        //Caso contrário, eles serão editáveis
        widget.getField('NMCANDIDATO').readOnly = !agrupadoPorLead;
        widget.getField('IDATIVO'    ).readOnly = !agrupadoPorLead;
        widget.getField('IDSTATUSCAN').readOnly = !agrupadoPorLead;
        widget.getField('DTINCLUSAO' ).readOnly = !agrupadoPorLead;
        
        // Escolhe qual data source será consultado
        if(agrupadoPorLead) {
            widget.dataSource.name = 'relatorioConsolidadoDeLeads_lead';
        } else {
            widget.dataSource.name = 'relatorioConsolidadoDeLeads_oper';
            
            // Remove os valores desses campos da current row
            currentRow.NMCANDIDATO = null;
            currentRow.IDATIVO = null;
            currentRow.DTINCLUSAO = null;
            currentRow.IDSTATUSCAN = null;
        }
        
        // Recarrega o filtro para aplicar as mudanças visualmente.
        widget.reload();
    };
    
    /**
     * Define quais colunas aparecerão na grid(list) com base no agrupador
     *selecionado.
     */
    this.definirVisibilidadeDosCampos = function(args) {
        var agrupadoPorLead = args.e.preference.FILTER_ROW.AGRUPADOR == 'L';
        var widget = args.owner;
        
        // Colunas visíveis quando agrupar por lead
        widget.getField('NMCANDIDATO').isVisible     = agrupadoPorLead;
        widget.getField('IDATIVO').isVisible         = agrupadoPorLead;
        widget.getField('IDSTATUSCAN').isVisible     = agrupadoPorLead;
        widget.getField('DTINCLUSAO').isVisible      = agrupadoPorLead;
        widget.getField('NMOPERRESP').isVisible      = agrupadoPorLead;
        widget.getField('DSSEGMER').isVisible        = agrupadoPorLead;
        widget.getField('QNTEVOL').isVisible         = agrupadoPorLead;
        widget.getField('QNTTAREFA').isVisible       = agrupadoPorLead;
        widget.getField('QNTINTER').isVisible        = agrupadoPorLead;
        
        // Colunas visíveis quando agrupar por operador
        widget.getField('NMOPERADOR').isVisible      = !agrupadoPorLead;
        widget.getField('CONTRATADOS').isVisible     = !agrupadoPorLead;
        widget.getField('ABERTOS').isVisible         = !agrupadoPorLead;
        widget.getField('QUALIFICADOS').isVisible    = !agrupadoPorLead;
        widget.getField('DESQUALIFICADOS').isVisible = !agrupadoPorLead;
        
        // Redesenha o cabeçalho da grid
        widget.redraw();
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('RelatorioConsolidadoDeLeadsController', RelatorioConsolidadoDeLeadsController);
});