function API_GoogleCalendar(ZHPromise, ScreenService, Messages) {
    
    // Array of API discovery doc URLs for APIs used by the quickstart
    var DISCOVERY_DOCS = ["https://www.googleapis.com/discovery/v1/apis/calendar/v3/rest"];
    
    // Authorization scopes required by the API; multiple scopes can be
    // included, separated by spaces.
    var SCOPES = "https://www.googleapis.com/auth/calendar";
    
    //
    var defaultEventId = "pebbiancrm";
    
    /**
     *  On load, called to load the auth2 library and API client library.
    */
    this.loadAuth2AndApiClientLibrary = function() {
        
        gapi.load('client:auth2', this.initClient);
        
    };
    
    /**
     *  Initializes the API client library and API key;
    */
    this.initClient = function(){
        
        gapi.client.init({
            discoveryDocs: DISCOVERY_DOCS,
            clientId: window.googleClientId,
            scope: SCOPES,
            apiKey: window.googleApiKey
        }).then(function(){
            gapi.client.setApiKey(window.googleApiKey);
        }.bind(this));
        
    };
    
    /**
     *  Sign in Google if there is not logged user.
    */
    this.signInGoogle = function() {
        
        var zhPromiseSignIn = ZHPromise.defer();
            
        if(!this.isLoggedSomeGoogleUser()){
            
            gapi.auth2.getAuthInstance().signIn().then(function(response){
                
                gapi.auth2.getAuthInstance().isSignedIn.listen(updateSigninStatus);
                zhPromiseSignIn.resolve();
                
            },function(response){
                
                var errorMessage;
                if(response.error === "popup_blocked_by_browser"){
                    errorMessage = Messages.POPUP_BLOQUEADA;
                } else {
                    errorMessage = Messages.FALHA_SINC_GOOGLE_CALENDAR;
                }
                
                zhPromiseSignIn.reject(errorMessage);
            });    
            
        }else{
            zhPromiseSignIn.resolve();
        }
        
        return zhPromiseSignIn.promise;
        
    };
    
    /**
     *  Called when the signed in status changes, to update the UI
     *  appropriately. After a sign-in, the API is called.
     * 
     * @param {Boolean} isSignedIn description
    */
    function updateSigninStatus(isSignedIn){
        
        if(!isSignedIn){
            
            var activeWidget = getActiveWidget();
            activeWidget.reload();
            
        }    
        
    }
    
    /**
     * Retorna a widget ativa.
    */
    function getActiveWidget() {
        
        return angular.element('#footer').scope().templateManager.container.getActiveWidget();
       
    }
    
    /**
     *  Sign out Google.
    */
    this.signOutGoogle = function () {
        gapi.auth2.getAuthInstance().signOut();
    };
    
    /**
     * Create an event(for logged user) according object received 
     * as parameter.
     * 
     * @param {object} commitmentModel object with the
     * commitment informations.
     * 
     * @objectModel = {TITLE: XXXX, INITDATE: XX/XX/XXXX, INITTIME: XX:XX, ENDDATE: XX/XX/XXXX, ENDTIME: XX:XX, DESCRIPTION: XXXX}
     * 
     * "ATTENTION WITH THE VARIABLES FORMAT".
    */
    this.addGoogleEvent = function(objectWithCommitmentInfos){
        var deferred = ZHPromise.defer();
        
        var event = this.createDefaultGoogleEventObject(objectWithCommitmentInfos);
        
        var request = gapi.client.calendar.events.insert({
            'calendarId': 'primary',
            'resource': event
        });
        
        if(this.isLoggedSomeGoogleUser()){
            request.execute(function(response){
                
                if(Boolean(response.error)){
                    deferred.reject();
                }else{
                    deferred.resolve();    
                }
                
            });
        }
        
        return deferred.promise;
    };
    
    /**
     * Returns a default google event.
    */
    this.createDefaultGoogleEventObject = function(event){
        
        var defaultEvent = {
            'id': defaultEventId + event.ID,
            'description': event.DESCRIPTION,
            'start': {
                'dateTime': this.createGoogleEventDateTimeFormat(event.INITDATE, event.INITTIME)
            },
            'end': {
                'dateTime': this.createGoogleEventDateTimeFormat(event.ENDDATE, event.ENDTIME)
            },
            'summary': event.TITLE
        };
        
        return defaultEvent;
    };
    
    /**
     * Returns a formatted string of the type "dateTime".
     * 
     * @param {string} date date required.
     * 
     * @param {string} time time required.
     * 
     * @return {string} formatted string of the type "dateTime".
    */
    this.createGoogleEventDateTimeFormat = function(date, time){
        var formattedDate = moment(date, 'DD/MM/YYYY').format('YYYY-MM-DD');
        var formattedTime = moment(time, 'HH:mm:ss').format('HH:mm:00');
        var formattedDateTime = formattedDate + 'T' + formattedTime + '-03:00';
        
        return formattedDateTime;
    };
    
    /**
     * Returns the logged Google user events.
     * 
     * @param {string} timeMax período máximo
     * para buscar os eventos.
     * 
     * @param {string} timeMin período mínimo
     * para buscar os eventos.
     * 
     * ATENÇÂO: Os parâmetros devem ser passados
     * do formato usado pelo google.
     * Ex.: 1997-05-22T09:00:00-03:00
    */
    this.getLoggedGoogleUserEvents = function(timeMax, timeMin) {
        
        var params = {
            'calendarId': 'primary'
        };
        
        if(timeMax){
            params.timeMax = timeMax;
        }
        if(timeMin){
            params.timeMin = timeMin;
        }
        
        return gapi.client.calendar.events.list(params)
        .then(function(response) {
            var events = response.result.items;
            return events;
        });
        
    };
    
    /**
     * Returns if exist some looged google user.
    */
    this.isLoggedSomeGoogleUser = function(){
        
        return gapi.auth2.getAuthInstance().isSignedIn.get();
        
    };
    
    /**
     * Delete a google event by received id.
    */
    this.deleteGoogleEvent = function(id){

        var params = {'calendarId': 'primary', 'eventId': defaultEventId + id};
        var request = gapi.client.calendar.events.delete(params);
        
        if(this.isLoggedSomeGoogleUser()){
            request.execute();
        }
        
    };
    
    /**
     * Edit a google event by received id in event object.
    */
    this.editGoogleEvent = function(event){
        
        var zhPromise = ZHPromise.defer();
        var defaultGoogleEvent = this.createDefaultGoogleEventObject(event);
        
        var params = {'calendarId': 'primary', 'eventId': defaultEventId + event.ID, 'resource': defaultGoogleEvent};
        var request = gapi.client.calendar.events.update(params);
        
        if(this.isLoggedSomeGoogleUser()){
            request.execute(function(){
                zhPromise.resolve();
            },function(fail){
                zhPromise.reject();
            });
        }
        
        return zhPromise.promise;
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('API_GoogleCalendar', API_GoogleCalendar);
});